(*********************************************************************)
(*                          Litmus                                   *)
(*                                                                   *)
(*        Luc Maranget, INRIA Paris-Rocquencourt, France.            *)
(*        Susmit Sarkar, University of Cambridge, UK.                *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique and the authors. All rights reserved.             *)
(*  This file is distributed  under the terms of the Lesser GNU      *)
(*  General Public License.                                          *)
(*********************************************************************)

module type I = sig
  module V : Constant.S
  type arch_reg
  val arch : Misc.arch
  val forbidden_regs : arch_reg list
  val pp_reg : arch_reg -> string
  val reg_compare : arch_reg -> arch_reg -> int
  val reg_to_string  : arch_reg -> string
  val internal_init : arch_reg -> (string * string) option
  val reg_class : arch_reg -> string
  val comment : char
end

module type S = sig
  module I : I

  val vToName : I.V.v -> string

  include Location.S 
  with type loc_reg = I.arch_reg and type loc_global = string
  module Out : Template.S
  with type arch_reg = I.arch_reg and type v = I.V.v
  module MapValue : Map.S with type key = I.V.v

(* A bit of state handling *)
  type state = (location * I.V.v) list

  val find_in_state : location -> state -> I.V.v

end

module Make(I:I) : S with module I = I
= struct
  module I = I

  let vToName v = match v with
  | I.V.Concrete i -> "addr_" ^ string_of_int i
  | I.V.Symbolic s -> s

  include Location.Make
      (struct
        include I

        type arch_global = string
        let maybev_to_global m = vToName (I.V.maybevToV m)
        let pp_global s = s
        let global_compare = String.compare
      end)

  module Out = Template.Make(I) (I.V)

  module MapValue =
    Map.Make
      (struct
        type t = I.V.v
        let compare = I.V.compare
      end)

(* A bit of state handling *)
  type state = (location * I.V.v) list

  let rec find_in_state loc = function
    | [] -> I.V.intToV 0
    | (loc2,v)::rem ->
        if location_compare loc loc2 = 0 then v
        else find_in_state loc rem

end
